
import java.io.*;
import java.sql.*;
import java.util.*;
import java.awt.*;

/*
 *
 * This java application handles adding stuff to the cart.
 * It first reads the content of a file named "cart" in the
 * current directory and outputs its content, before appending the
 * passed item at the bottom of the form.
 *
 */
public class Cart {

    private static PrintWriter cout;
    private static BufferedReader cin;
    private static final char underscore = '_';
    private static final char space = ' ';

    public static void main(String[] args) throws IOException {

		// The cart file may be affected by the arguments passed:
		// if 0 arguments are passed, then no shoe is deleted or inserted
		// if 1 argument is passed, this argument is the shoe to be deleted
		// if 3 arguments are passed, these arguments are shoe to be inserted
		switch( args.length ) {

			// no shoe is deleted or inserted--just build the page using current shoes in the cart
			case 0: buildPage();
					break;

			// delete the shoe represented by the argument from cart, then build the page
			case 1: deleteSelection( args[0] );
					buildPage();
					break;

			// insert the shoe represented by the argument to cart, then build the page
			case 3: if( addToCart(args[0], args[1], args[2]) ){
						buildPage();
					}
					else {
						buildErrorPage();
					}
					break;

			// the default should never be reached--if it is, this is an unrecoverable error
			default: 	System.err.println( "Bad arguments to main method of Cart" );
						System.exit(1);

    	} // end switch

	} // end method main


	private static void deleteSelection( String delShoe ) throws IOException {

		// create deleteShoe
		Shoe deleteShoe = Shoe.convertStringToShoe( delShoe, "#" );
		Vector updatedShoeList = new Vector();
		Shoe curShoe;

		cin = new BufferedReader(new FileReader("cart"));
		// read in each shoe from the file 'cart' and possibly put it into updatedShoeList
		while (cin.ready()) {

			// read in a shoe from the file
			curShoe = Shoe.convertStringToShoe( cin.readLine(), "\t" );

			// if the shoe just read in is  the shoe to be deleted (delShoe),
			// then add curShoe to updatedShoeList
			// will be changed to match both name and size
			if ( !(curShoe.equals(deleteShoe) ) ){

				updatedShoeList.addElement( curShoe );

			}

			// otherwise, curShoe DOES match the shoe to be deleted (delShoe),
			// either decrement its qty (if qty > 1) or remove it altogether (if qty == 1)
			// NOTE:  this assumes that the file has NO DUPLICATE SHOES ( i.e., shoes with the
			//        same name, and eventually size--we will eventually add checking for size to
			//        our current check for just name.
			else {

				// if curShoe's qty > 1, decrement its qty by 1 and add to updatedShoeList
				if( curShoe.getQty() > 1 ) {

					curShoe.decrQty();
					updatedShoeList.addElement( curShoe );

				}

				// otherwise, curShoe's qty == 1, so don't add it to the updatedShoeList
				// (this is essentially decrementing qty to a value of 0
			}
		}

		cin.close();

		// write out the updatedShoeList to the file 'cart' (which overwrites the old file)
		writeToCart( updatedShoeList );
	}


	private static boolean addToCart(String addShoe, String addShoeSize, String qtyStr) throws IOException {

		int addShoeQty;
		try {
			addShoeQty = Integer.parseInt( qtyStr );
			if( addShoeQty < 1 ) return false;  // don't want to add 0 or negative # of shoes
		}

		// if user inputs a non-integer for quantity, don't add any shoes to cart
		catch( NumberFormatException nfe ) {
			return false;
		}

		// extract values for addShoe
		StringTokenizer st = new StringTokenizer(addShoe, "#");

		Shoe curShoe;

		// name brand type style price qty
		Shoe addedShoe = new Shoe( st.nextToken(), (st.nextToken()).replace( underscore, space),
								   st.nextToken(), (st.nextToken()).replace( underscore, space),
								   Double.parseDouble( st.nextToken() ),
								   Double.parseDouble( addShoeSize ), addShoeQty );


		String sqlExprGetQty = new String( "SELECT qty FROM size_qty WHERE name=\"" +
											addedShoe.getName() + "\" AND size=\"" +
											addedShoe.getSize() + "\";" );
		int curQty = Integer.parseInt(DBConnect.execSQLquery( sqlExprGetQty, 1 ));

		if( curQty < addShoeQty ) {
			return false;
		}

		boolean matchExists = false;
		Vector updatedShoeList = new Vector();
		cin = new BufferedReader(new FileReader("cart"));

		// read in each shoe from the file 'cart' and possibly put it into updatedShoeList
		while (cin.ready()) {

			// read in a shoe from the file
			curShoe = Shoe.convertStringToShoe( cin.readLine(), "\t" );

			// see if curShoe matches addShoe
			// if so, increment curShoe's qty by addShoe's qty
			// otherwise, just add addShoe to updatedShoeList

			if( addedShoe.equals( curShoe ) ) {
				curShoe.setQty( curShoe.getQty() + addedShoe.getQty() );
				matchExists = true;
			}

			updatedShoeList.addElement( curShoe );

		}
		if( !matchExists ) {
			updatedShoeList.addElement( addedShoe );
		}

		cin.close();

		// write out the updatedShoeList to the file 'cart' (which overwrites the old file)
		writeToCart( updatedShoeList );

		return true;

    } // end method addToCart


	private static void writeToCart( Vector shoes ) throws IOException {

		cout = new PrintWriter(new BufferedWriter( new FileWriter ( "cart" ) ) );
		for ( int i = 0; i < shoes.size(); i++ ) {
			cout.println( ((Shoe)shoes.elementAt( i )).toString("\t") );
		}
		cout.close();

	} // end writeToCart method


	private static void buildPage() throws IOException {

		String font5 = "<font face=\"Bauhaus 93\" color=\"#008000\" size=\"5\">";


		System.out.println("Content-type: text/html\n\n");
		System.out.println("<html>");
		System.out.println("<head>");
		System.out.println("<title> Shopping Cart</title>");
		System.out.println("</head>");
		System.out.println("<body>");

		System.out.println( "<table align=\"center\" width=\"100%\" cellpadding=\"6\">" );

	// banner
		System.out.println( "<tr align=\"center\">" );
		System.out.println( "<td colspan=\"2\">" );
		System.out.println( "<i><img src=\"shoepic1.bmp\" width=\"100\" height=\"100\"></i>" );
		System.out.println( "<i><img src=\"E-Shoe1.gif\"  width=\"300\" height=\"100\"></i>" );
		System.out.println( "<i><img src=\"shoepic2.bmp\" width=\"100\" height=\"100\"></i>" );
		System.out.println( "</td>" );
		System.out.println( "</tr>" );

	// First Thick Line
		System.out.println( "<tr align=\"center\">" );
		System.out.println( "<td colspan=\"2\">" );
		System.out.println( "<i><img src=\"line30.gif\"  width=\"100%\"></i>" );
		System.out.println( "</td>" );
		System.out.println( "</tr>" );


		System.out.println("<tr>" );
		System.out.println("<td align=\"center\"><i><img src=\"cart.gif\"></i></td></tr>");
		System.out.println("</table>" );
		System.out.println("<form method=post action=\"http://tant14.cs.wisc.edu/cart.pl\">");

		System.out.println( "<table align=\"center\" width=\"80%\" border=\"1\" cellpadding=\"4\">" );

		System.out.println( "<tr>" );
		System.out.println( "<td><input type=\"submit\" value=\"Decrease QTY by 1\" " +
                           "name=\"delete\"></td>" );
        System.out.println( "<td width=\"100\" align=\"center\">" + font5 + "QTY</font></td>" );
		System.out.println( "<td width=\"100\" align=\"center\">" + font5 + "NAME</font></td>" );
		System.out.println( "<td width=\"100\" align=\"center\">" + font5 + "BRAND</font></td>" );
		System.out.println( "<td width=\"100\" align=\"center\">" + font5 + "TYPE</font></td>" );
		System.out.println( "<td width=\"100\" align=\"center\">" + font5 + "STYLE</font></td>" );
		System.out.println( "<td width=\"100\" align=\"center\">" + font5 + "PRICE</font></td>" );
		System.out.println( "<td width=\"100\" align=\"center\">" + font5 + "SIZE</font></td>" );
		System.out.println( "</tr>" );

		Vector shoes = Shoe.readShoesFromFile( "cart" );

		//while (cin.ready()) {
		for( int i = 0; i < shoes.size(); i++ ) {

		    Shoe shoe = (Shoe)(shoes.elementAt(i));
			System.out.println( "<tr>" );
			System.out.println( "<td align=\"center\"><input type=\"radio\" name=\"line" +
   	                                "\" value=\"" + shoe.toString() + "\"></td>" );
   	        System.out.println( "<td align=\"center\">" + shoe.getQty() + "</td>" );
   	        System.out.println( "<td align=\"center\">" + shoe.getName() + "</td>" );
   	        System.out.println( "<td align=\"center\">" + shoe.getBrand() + "</td>" );
   	        System.out.println( "<td align=\"center\">" + shoe.getType() + "</td>" );
   	        System.out.println( "<td align=\"center\">" + shoe.getStyle() + "</td>" );
   	        System.out.println( "<td align=\"center\">" + shoe.getPrice() + "</td>" );
   	        System.out.println( "<td align=\"center\">" + shoe.getSize() + "</td>" );
   	        System.out.println( "</tr>" );
		}

		System.out.println( "</table>" );

		System.out.println( "<p>" );

		System.out.println( "<table align=\"center\" width=\"100%\" cellpadding=\"6\">" );
		// Second thick line
		System.out.println( "<tr align=\"center\">" );
		System.out.println( "<td colspan=\"3\">" );
		System.out.println( "<i><img src=\"line30.gif\"  align=\"center\" width=\"100%\"></i>" );
		System.out.println( "</td>" );
		System.out.println( "</tr>" );


		System.out.println("<tr>" );

		// the continue shopping button refers the customer to the main page
		System.out.println("<td align=\"center\"><a href=\"http://tant14.cs.wisc.edu/eShoeScript.pl\">" +
                           "<img src=\"continue-shopping.gif\"></i>" + "</a></td>");

		System.out.println("</tr><tr>" );

		// the check_out button refers the customer to the check_out page
		System.out.println("<td align=\"center\"><a href=\"http://tant14.cs.wisc.edu/check_out.pl\">" +
	                       "<img src=\"checkout.gif\"></i>" + "</a></td>");

	    System.out.println( "</tr>" );
	    System.out.println( "</table>" );

		System.out.println("</form>");
		System.out.println("</body>");
		System.out.println("</html>");

		cin.close();

	} // end method buildPage


	// page that is built if erroroneous quantity entered
	private static void buildErrorPage() throws IOException {

		String font5 = "<font face=\"Bauhaus 93\" color=\"#008000\" size=\"5\">";


		System.out.println("Content-type: text/html\n\n");
		System.out.println("<html>");
		System.out.println("<head>");
		System.out.println("<title> Shopping Cart</title>");
		System.out.println("</head>");
		System.out.println("<body>");

		System.out.println( "<table align=\"center\" width=\"100%\" cellpadding=\"6\">" );

	// banner
		System.out.println( "<tr align=\"center\">" );
		System.out.println( "<td colspan=\"2\">" );
		System.out.println( "<i><img src=\"shoepic1.bmp\" width=\"100\" height=\"100\"></i>" );
		System.out.println( "<i><img src=\"E-Shoe1.gif\"  width=\"300\" height=\"100\"></i>" );
		System.out.println( "<i><img src=\"shoepic2.bmp\" width=\"100\" height=\"100\"></i>" );
		System.out.println( "</td>" );
		System.out.println( "</tr>" );

	// First Thick Line
		System.out.println( "<tr align=\"center\">" );
		System.out.println( "<td colspan=\"2\">" );
		System.out.println( "<i><img src=\"line30.gif\"  width=\"100%\"></i>" );
		System.out.println( "</td>" );
		System.out.println( "</tr>" );


		System.out.println("<tr>" );
		System.out.println("<td align=\"center\"><i><img src=\"cart.gif\"></i></td></tr>");
		System.out.println("</table>" );
		System.out.println("<form method=post action=\"http://tant14.cs.wisc.edu/eShoeScript.pl\">");

		System.out.println( "<table align=\"center\" width=\"80%\" cellpadding=\"8\">" );

		System.out.println( "<tr>" );
		System.out.println( "<td align=\"center\">" + font5 );
		System.out.println( "ERROR with quantity chosen (either too much or bad value entered!)");
		System.out.println( "</font></td>" );
		System.out.println( "</tr>" );

		System.out.println( "<tr>" );

		// the continue shopping button refers the customer to the main page
		System.out.println("<td align=\"center\"><a href=\"http://tant14.cs.wisc.edu/eShoeScript.pl\">" +
                           "<img src=\"continue-shopping.gif\"></i>" + "</a></td>");
        System.out.println( "</tr>" );
		System.out.println( "</table>" );
		System.out.println("</form>");
		System.out.println("</body>");
		System.out.println("</html>");

	} // end of method buildErrorPage

} // end class Cart